#!/usr/bin/python
# coding=utf-8

# Buzz Grid - an abstract arcade game in HTML5
# 2013-05-14 Felix Pleșoianu <felix@plesoianu.ro>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from math import trunc
import random
import pygame
import os.path

BLACK = (0, 0, 0)
GREEN = (0, 204, 0)
GREEN2 = (102, 204, 102)
MAGENTA = (102, 0, 102)
BLUE = (0, 0, 255)
CYAN = (0, 255, 255)
YELLOW = (255, 255, 0)
RED = (255, 0, 0)
RED2 = (255, 102, 102)
ORANGE = (255, 204, 0)
WHITE = (255, 255, 255)

roamer_bitmap = [
	bytearray([0, 0, 0, 0, 0, 0, 1, 1]),
	bytearray([0, 0, 0, 0, 1, 1, 1, 0]),
	bytearray([0, 1, 1, 1, 1, 1, 0, 0]),
	bytearray([0, 0, 1, 1, 1, 0, 0, 0]),
	bytearray([0, 0, 0, 1, 1, 1, 0, 0]),
	bytearray([0, 0, 1, 1, 1, 1, 1, 0]),
	bytearray([0, 1, 1, 1, 0, 0, 0, 0]),
	bytearray([1, 1, 0, 0, 0, 0, 0, 0])]
prize_bitmap = [
	bytearray([0, 0, 1, 1, 1, 1, 0, 0]),
	bytearray([0, 1, 1, 1, 1, 1, 1, 0]),
	bytearray([1, 1, 1, 1, 0, 0, 1, 1]),
	bytearray([1, 1, 1, 0, 1, 1, 1, 1]),
	bytearray([1, 1, 1, 1, 0, 1, 1, 1]),
	bytearray([1, 1, 0, 0, 1, 1, 1, 1]),
	bytearray([0, 1, 1, 1, 1, 1, 1, 0]),
	bytearray([0, 0, 1, 1, 1, 1, 0, 0])]
trap_bitmap = [
	bytearray([0, 1, 1, 0, 1, 1, 1, 0]),
	bytearray([1, 1, 1, 1, 0, 1, 1, 1]),
	bytearray([1, 1, 1, 0, 1, 1, 1, 1]),
	bytearray([0, 1, 0, 1, 0, 1, 0, 1]),
	bytearray([1, 0, 1, 0, 1, 0, 1, 0]),
	bytearray([1, 1, 1, 1, 0, 1, 1, 1]),
	bytearray([1, 1, 1, 0, 1, 1, 1, 1]),
	bytearray([0, 1, 1, 1, 0, 1, 1, 0])]
ship_bitmap = [
	bytearray([1, 1, 1, 0, 0, 1, 1, 1]),
	bytearray([1, 1, 0, 0, 0, 0, 1, 1]),
	bytearray([1, 0, 1, 0, 0, 1, 0, 1]),
	bytearray([0, 0, 0, 0, 0, 0, 0, 0]),
	bytearray([0, 0, 0, 0, 0, 0, 0, 0]),
	bytearray([1, 0, 1, 0, 0, 1, 0, 1]),
	bytearray([1, 1, 0, 0, 0, 0, 1, 1]),
	bytearray([1, 1, 1, 0, 0, 1, 1, 1])]

levels = [
	{"map": [[-4, -4, 4, 4]], "prizes": 16, "traps": 16, "target": 12},
	{"map": [[-3, -3, 3, 3],
		[-5, -5, -1, -1],
		[1, 1, 5, 5],
		[-5, 1, -1, 5],
		[1, -5, 5, -1]], "prizes": 20, "traps": 20, "target": 15},
	{"map": [[-1, -5, 1, 5],
		[-5, -5, -3, 5],
		[3, -5, 5, 5],
		[-5, -1, 5, 1],
		[-5, -5, 5, -3],
		[-5, 3, 5, 5]], "prizes": 23, "traps": 23, "target": 17},
	{"map": [[-7, -2, -3, 2],
		[-2, -7, 2, -3],
		[3, -2, 7, 2],
		[-2, 3, 2, 7],
		[-3, -1, 3, 1],
		[-1, -3, 1, 3]], "prizes": 24, "traps": 24, "target": 18},
	{"map": [[-3, -3, 3, 3],
		[-6, -2, 6, 2],
		[-8, -1, 8, 1],
		[-2, -6, 2, 6],
		[-1, -8, 1, 8]], "prizes": 27, "traps": 27, "target": 21},
	{"map": [[-2, -2, 2, 2],
		[-6, -2, -2, 10],
		[2, -2, 6, 10],
		[-2, 6, 2, 10]], "prizes": 32, "traps": 32, "target": 24},
	{"map": [[-2, -7, 6, -5],
		[-6, -4, 2, 0],
		[3, -5, 6, 0],
		[-6, 0, -3, 5],
		[-2, 0, 6, 4],
		[-6, 5, 2, 7]], "prizes": 33, "traps": 33, "target": 25}
]
	
speeds = [
	{"ship": 48, "roamer": 30, "roamer_timer": 3},
	{"ship": 60, "roamer": 40, "roamer_timer": 3},
	{"ship": 75, "roamer": 50, "roamer_timer": 2},
	{"ship": 100, "roamer": 60, "roamer_timer": 2},
	{"ship": 120, "roamer": 80, "roamer_timer": 1}]

resolution = 1200
fps = 20

ship_speed = 48 # MUST be a divisor of resolution.
roamer_speed = 30  # MUST be a divisor of resolution.
roamer_timer = 3
muted = False

explosion_duration = 3
death_timeout = 3
level_timeout = 3
initial_lives = 2
lives_per_level = 1

directions = ["n", "s", "e", "w"]

def set_game_speed(speed):
	global ship_speed, roamer_speed, roamer_timer
	if speed < 1 or speed > len(speeds):
		raise "Invalid speed: " + str(speed)
	ship_speed = speeds[speed - 1]["ship"]
	roamer_speed = speeds[speed - 1]["roamer"]
	roamer_timer = speeds[speed - 1]["roamer_timer"]
	
def is_on_map(x, y, the_map):
	for m in the_map:
		if x >= m[0] and y >= m[1] and x <= m[2] and y <= m[3]:
			return True
	return False
	
def unique(elements):
	return list(set(elements))

def collect_nodes(r1, c1, r2, c2):
	nodes = []
	for i in range(r1, r2 + 1):
		for j in range(c1, c2 + 1):
			nodes.append((i * resolution, j * resolution))
	return nodes

def unique_nodes(the_map):
	nodes = []
	for m in the_map:
		nodes.extend(apply(collect_nodes, m))
	return unique(nodes)
	
def ai_dumb(self):
	self.facing = random.choice(directions)
	self.speed = roamer_speed

class Thing:
	def __init__(self, type, x = 0, y = 0):
		self.type = type
		self.x = x
		self.y = y
		
		if self.type == "ship":
			self.size = resolution * 4 / 10
		elif self.type == "prize":
			self.size = resolution / 4
		elif self.type == "trap":
			self.size = resolution / 4
		elif self.type == "roamer":
			self.size = resolution * 4 / 10
	
	def within(self, thing):
		if abs(self.x - thing.x) <= thing.size:
			return abs(self.y - thing.y) <= thing.size
		else:
			return False
	
	def touches(self, thing):
		sum_of_sizes = self.size + thing.size
		if abs(self.x - thing.x) <= sum_of_sizes:
			return abs(self.y - thing.y) <= sum_of_sizes
		else:
			return False

class Mob(Thing):
	def __init__(self, type, x = 0, y = 0):
		Thing.__init__(self, type, x, y)
		self.speed = 0
		self.heading = "n"
		self.facing = "n"

	def update(self, level):
		new_x = self.x
		new_y = self.y
		if new_x % resolution == 0 and new_y % resolution == 0:
			if hasattr(self, "ai"):
				self.ai(self)
			self.heading = self.facing

		if self.heading == "n":
			new_y += self.speed
		elif self.heading == "s":
			new_y -= self.speed
		elif self.heading == "e":
			new_x += self.speed
		elif self.heading == "w":
			new_x -= self.speed
		else:
			raise "Bad heading: " + self.heading

		if is_on_map(new_x, new_y, level.map):
			self.x = new_x
			self.y = new_y
	

class GameWorld:
	high_score = 0
	
	level = 0
	score = 0
	lives = initial_lives
	game_speed = 1

	death_timer = 0
	level_timer = 0

	ship = Mob("ship")
	
	def init_level(self, number, data):
		self.ship.x = 0
		self.ship.y = 0
		self.ship.facing = 'n'
		self.ship.speed = 0
		
		self.level = number
		self.lives += lives_per_level
		self.target = data["target"]
		
		self.death_timer = 0
		self.level_timer = 0

		# self.map = []
		#for i in range(len(data["map"])):
		#	self.map.append([0, 0, 0, 0])
		#	for j in range(4):
		#		self.map[i][j] = data["map"][i][j] * resolution
		self.map = [[j * resolution for j in i] for i in data["map"]]
		
		self.nodes = unique_nodes(data["map"])
		random.shuffle(self.nodes)
		self.prizes = []
		for i in range(data["prizes"]):
			x, y = self.nodes.pop()
			if x == 0 and y == 0: continue
			self.prizes.append(Thing("prize", x, y))
		self.traps = []
		for i in range(data["traps"]):
			x, y = self.nodes.pop()
			if (x == 0 and y == 0): continue
			self.traps.append(Thing("trap", x, y))
		
		self.roamers = []
		self.roamer_queue = []
		self.explosions = []
	
	def update(self, delta_t):
		self.handle_collisions()
		for i in self.roamers:
			i.update(self)
			
		rq = self.roamer_queue
		for i in rq:
			i["timer"] -= delta_t
		if len(rq) > 0 and rq[0]["timer"] <= 0:
			mob = Mob("roamer", rq[0]["x"], rq[0]["y"])
			mob.ai = ai_dumb
			self.roamers.append(mob)
			del rq[0]
			if not muted: thunk_sound.play();
		
		xpl = self.explosions
		for i in xpl:
			i["age"] += delta_t
		if len(xpl) > 0:
			if xpl[0]["age"] >= explosion_duration:
				del xpl[0]

		if self.death_timer > 0:
			self.death_timer -= delta_t
		elif self.lives <= 0:
			self.level = 0 # Game over
		elif self.level_timer > 0:
			self.level_timer -= delta_t
			if self.level_timer <= 0: self.next_level()
		
		self.ship.update(self)
	
	def _roamer_vs_trap(self):
		for roamer in self.roamers:
			for trap in self.traps:
				if not roamer.within(trap): continue
				self.roamers.remove(roamer)
				self.traps.remove(trap)
				self.explosions.append({
					"x": roamer.x,
					"y": roamer.y,
					"text": "Bzzzt!",
					"age": 0
				})
				if not muted: bzzzt_sound.play()
				return

	def handle_collisions(self):
		self._roamer_vs_trap()

		if self.death_timer > 0: return
		if self.level_timer > 0: return

		for roamer in self.roamers:
			if self.ship.within(roamer):
				self.roamers.remove(roamer)
				self.explosions.append({
					"x": roamer.x,
					"y": roamer.y,
					"text": "Zap!",
					"age": 0
				})
				self.lives -= 1
				self.death_timer = death_timeout
				if not muted: bzzzt_sound.play()
				break
		
		for trap in self.traps:
			if self.ship.touches(trap):
				self.traps.remove(trap)
				self.explosions.append({
					"x": self.ship.x,
					"y": self.ship.y,
					"text": "Crash!",
					"age": 0
				})
				self.lives -= 1
				self.death_timer = death_timeout
				if not muted: crash_sound.play()
		
		for prize in self.prizes:
			if not prize.within(self.ship): continue
			self.prizes.remove(prize)
			self.score += self.game_speed
			if self.score > self.high_score:
				self.high_score = self.score
			self.target -= 1
			if len(self.prizes) <= 0:
				self.level_complete()
			self.roamer_queue.append({
				"x": prize.x,
				"y": prize.y,
				"timer": roamer_timer
			})
			self.explosions.append({
				"x": prize.x,
				"y": prize.y,
				"text": "Ding!",
				"age": 0
			});
			if not muted: ding_sound.play()
			break
	
	def level_complete(self):
		self.level_timer = level_timeout
		self.explosions.append({
			"x": self.ship.x,
			"y": self.ship.y,
			"text": "Zoom!",
			"age": 0
		})
		if not muted: zoom_sound.play()
	
	def next_level(self):
		self.level += 1
		if self.level > len(levels):
			self.level = 1
			if self.game_speed < len(speeds):
				self.game_speed += 1
			set_game_speed(self.game_speed)
		self.init_level(self.level, levels[self.level - 1])
	
class GameScreen:
	def __init__(self, model, screen):
		self.model = model
		
		self.screen = screen
		self.width, self.height = screen.get_size()
		self.bglayer = pygame.Surface(screen.get_size())

		font_name = pygame.font.get_default_font()
		self.title_font = pygame.font.SysFont(
			font_name, self.height / 5)
		self.hud_font = pygame.font.SysFont(
			font_name, self.height / 10)
		self.msg_font = pygame.font.SysFont(
			font_name, self.height / 20)
		self.pause_text = self.hud_font.render(
			"Game paused", False, WHITE)
		self.skip_text = self.hud_font.render(
			"[L]evel skip unlocked", False, WHITE)
		self.help_text = self.hud_font.render(
			"Arrows or W, A, S, D to move", False, WHITE)
				
		self.game_paused = False

		self.render_backround()
		
	def render(self):
		m = self.model
		
		self.screen.blit(self.bglayer, (0, 0))

		if m.level == 0:
			self.render_title_screen()
			# Hide the [P]ause text.
			self.screen.fill(MAGENTA, (625, 540, 75, 30))
		else:
			self.render_game_world()
			# Hide the [P]lay text.
			self.screen.fill(MAGENTA, (625, 510, 75, 30))
			if self.game_paused:
				self.render_status(self.pause_text)
			elif model.target <= 0:
				self.render_status(self.skip_text)
			elif model.ship.speed == 0:
				self.render_status(self.help_text)

			if model.death_timer > 0 and model.lives <= 0:
				self.render_message("GAME", "OVER")
			elif model.level_timer > 0:
				self.render_message("LEVEL", "COMPLETE")
		
		text = self.hud_font.render(str(m.level), False, GREEN)
		self.screen.blit(text, (625, 60))
		text = self.hud_font.render(str(m.lives), False, GREEN)
		self.screen.blit(text, (625, 60 * 3))
		text = self.hud_font.render(str(m.score), False, GREEN)
		self.screen.blit(text, (625, 60 * 5))
		text = self.hud_font.render(str(m.high_score), False, GREEN)
		self.screen.blit(text, (625, 60 * 7))
		
		pygame.display.flip()
		
	def render_backround(self):
		self.bglayer.fill(BLACK)
		self.bglayer.fill(MAGENTA, (600, 0, 200, 600))
		
		msg = ["Level", "Lives", "Score", "Record"]
		for i in range(4):
			text = self.hud_font.render(msg[i], False, GREEN)
			self.bglayer.blit(text, (625, i * 120))
		
		text = self.msg_font.render("[P]lay", False, WHITE)
		self.bglayer.blit(text, (625, 510))
		text = self.msg_font.render("[P]ause", False, WHITE)
		self.bglayer.blit(text, (625, 540))
		text = self.msg_font.render("[M]ute", False, WHITE)
		self.bglayer.blit(text, (725, 510))
		text = self.msg_font.render("[Q]uit", False, WHITE)
		self.bglayer.blit(text, (725, 540))
			
	def render_title_screen(self):
		text = self.title_font.render("BUZZ", False, BLUE)
		self.screen.blit(text, text.get_rect(midbottom = (300, 300)))
		text = self.title_font.render("GRID", False, CYAN)
		self.screen.blit(text, text.get_rect(midtop = (300, 300)))
		
		text = self.msg_font.render("No Time To Play", False, GREEN)
		self.screen.blit(text, text.get_rect(midbottom = (300, 150)))
		text = self.msg_font.render("presents", False, GREEN)
		self.screen.blit(text, text.get_rect(midtop = (300, 150)))
		
		speed = "Speed: [-] " + str(self.model.game_speed) + " [+]"
		text = self.msg_font.render(speed, False, GREEN)
		self.screen.blit(text, text.get_rect(center = (300, 450)))

	def render_game_world(self):
		m = self.model

		self.screen.set_clip((0, 0, 600, 600))
			
		self.render_grid()

		for i in m.prizes:
			self.render_prize(i)
		for i in m.traps:
			self.render_trap(i)
		#for i in m.roamer_queue:
		#	self.render_countdown(i)
		for i in m.roamers:
			self.render_roamer(i)

		facing = self.model.ship.facing
		if self.model.death_timer > 0:
			line_width = 2
		else:
			line_width = 0
		if m.lives <= 0 or m.level_timer > 0:
			pass
		elif facing == "n":
			self.render_ship_n(line_width)
		elif facing == "e":
			self.render_ship_e(line_width)
		elif facing == "s":
			self.render_ship_s(line_width)
		elif facing == "w":
			self.render_ship_w(line_width)

		for i in m.explosions:
			self.render_explosion(i)

		self.screen.set_clip(None)
		
	def transform(self, x, y):
		ship = self.model.ship
		return (x / 10 + 300 - ship.x / 10,
			600 - (y / 10 + 300 - ship.y / 10))
		
	def render_grid(self):
		ship = self.model.ship
		for m in self.model.map:
			x1 = m[0] / 10 + 300 - ship.x / 10
			y1 = m[1] / 10 + 300 - ship.y / 10
			x2 = m[2] / 10 + 300 - ship.x / 10
			y2 = m[3] / 10 + 300 - ship.y / 10
			for j in range(x1, x2 + 1, resolution / 10):
				pygame.draw.line(self.screen,
					BLUE, (j, 600 - y1), (j, 600 - y2), 12)
				pygame.draw.line(self.screen,
					CYAN, (j, 600 - y1), (j, 600 - y2), 4)
			for j in range(y1, y2 + 1, resolution / 10):
				pygame.draw.line(self.screen,
					BLUE, (x1, 600 - j), (x2, 600 - j), 12)
				pygame.draw.line(self.screen,
					CYAN, (x1, 600 - j), (x2, 600 - j), 4)

	def render_prize(self, prize):
		pygame.draw.circle(
			self.screen,
			GREEN,
			self.transform(prize.x, prize.y),
			prize.size / 10)
		pygame.draw.circle(
			self.screen,
			GREEN2,
			self.transform(prize.x - 15, prize.y + 15),
			prize.size / 20)
	
	def render_trap(self, trap):
		bbox = pygame.Rect(0, 0, trap.size / 5, trap.size / 5)
		bbox.center = self.transform(trap.x, trap.y)
		pygame.draw.rect(self.screen, RED, bbox)
		bbox = pygame.Rect(0, 0, trap.size / 10, trap.size / 10)
		bbox.center = self.transform(trap.x - 15, trap.y + 15)
		pygame.draw.rect(self.screen, RED2, bbox)
	
	#def render_countdown(self, countdown):
	#	coords = self.transform(countdown["x"], countdown["y"])
	#	time = str(trunc(countdown["timer"]))
	#	pygame.draw.circle(self.screen, MAGENTA, coords, 30)
	#	text = self.hud_font.render(time, False, GREEN)
	#	self.screen.blit(text, text.get_rect(center = coords))
	
	def render_roamer(self, roamer):
		x, y, size = roamer.x, roamer.y, roamer.size
		pygame.draw.polygon(self.screen, ORANGE,
			[self.transform(x + size, y + size),
			self.transform(x - size, y - size),
			self.transform(x + size / 2, y - size / 2),
			self.transform(x - size / 2, y + size / 2),
			self.transform(x + size, y + size)])
		
	def render_ship_n(self, line_width):
		pygame.draw.polygon(self.screen, YELLOW,
			[(0 + 300, -48 + 300),
			(48 + 300, 24 + 300),
			(-48 + 300, 24 + 300),
			(0 + 300, -48 + 300)],
			line_width)
		pygame.draw.polygon(self.screen, YELLOW,
			[(0 + 300, -24 + 300),
			(48 + 300, 48 + 300),
			(-48 + 300, 48 + 300),
			(0 + 300, -24 + 300)],
			line_width)
		
	def render_ship_e(self, line_width):
		pygame.draw.polygon(self.screen, YELLOW,
			[(48 + 300, 0 + 300),
			(-24 + 300, 48 + 300),
			(-24 + 300, -48 + 300),
			(48 + 300, 0 + 300)],
			line_width)
		pygame.draw.polygon(self.screen, YELLOW,
			[(24 + 300, 0 + 300),
			(-48 + 300, 48 + 300),
			(-48 + 300, -48 + 300),
			(24 + 300, 0 + 300)],
			line_width)
		
	def render_ship_s(self, line_width):
		pygame.draw.polygon(self.screen, YELLOW,
			[(0 + 300, 48 + 300),
			(48 + 300, -24 + 300),
			(-48 + 300, -24 + 300),
			(0 + 300, 48 + 300)],
			line_width)
		pygame.draw.polygon(self.screen, YELLOW,
			[(0 + 300, 24 + 300),
			(48 + 300, -48 + 300),
			(-48 + 300, -48 + 300),
			(0 + 300, 24 + 300)],
			line_width)
		
	def render_ship_w(self, line_width):
		pygame.draw.polygon(self.screen, YELLOW,
			[(-48 + 300, 0 + 300),
			(24 + 300, 48 + 300),
			(24 + 300, -48 + 300),
			(-48 + 300, 0 + 300)],
			line_width)
		pygame.draw.polygon(self.screen, YELLOW,
			[(-24 + 300, 0 + 300),
			(48 + 300, 48 + 300),
			(48 + 300, -48 + 300),
			(-24 + 300, 0 + 300)],
			line_width)
	
	def render_explosion(self, expl):
		if expl["text"] == "Ding!":
			data = prize_bitmap
			color = GREEN
		elif expl["text"] == "Bzzzt!":
			data = trap_bitmap
			color = RED
		elif expl["text"] == "Zap!":
			data = roamer_bitmap
			color = ORANGE
		elif expl["text"] == "Zoom!":
			data = ship_bitmap
			color = YELLOW
		elif expl["text"] == "Crash!":
			data = trap_bitmap
			color = RED
		else:
			return

		age = expl["age"]
		for y in range(8):
			for x in range(8):
				if data[y][x] == 0: continue
				x1 = expl["x"] + (x - 4) * (100 * age)
				y1 = expl["y"] + (7 - y - 4) * (100 * age)
				x1, y1 = self.transform(x1, y1)
				x1 += random.randint(-2, 2)
				y1 += random.randint(-2, 2)
				self.screen.fill(color, (x1, y1, 10, 10))
	
	def render_status(self, text):
		self.screen.blit(text, text.get_rect(midbottom = (300, 600)))
	
	def render_message(self, text1, text2):
		t1 = self.title_font.render(text1, False, GREEN)
		t2 = self.title_font.render(text2, False, GREEN)
		self.screen.blit(t1, t1.get_rect(midbottom = (300, 300)))
		self.screen.blit(t2, t2.get_rect(midtop = (300, 300)))

pygame.mixer.pre_init(frequency = 44100)
pygame.init()

screen = pygame.display.set_mode((800, 600))
clock = pygame.time.Clock()
model = GameWorld()
renderer = GameScreen(model, screen)

pygame.display.set_caption("Buzz Grid Desktop Edition")
pygame.event.set_allowed([pygame.KEYDOWN, pygame.QUIT,
	pygame.JOYHATMOTION, pygame.JOYBUTTONDOWN])

if pygame.joystick.get_count() > 0:
	joystick = pygame.joystick.Joystick(0)
	joystick.init()
	buttons = joystick.get_numbuttons();
else:
	joystick = None
	buttons = 0;

bzzzt_sound = pygame.mixer.Sound(os.path.join("sound", "thunk.wav"))
ding_sound = pygame.mixer.Sound(os.path.join("sound", "ding2.wav"))
thunk_sound = pygame.mixer.Sound(os.path.join("sound", "bzzzt.wav"))
zoom_sound = pygame.mixer.Sound(os.path.join("sound", "zoom3.wav"))
crash_sound = pygame.mixer.Sound(os.path.join("sound", "crash.wav"))

running = True
while running:
	delta_t = clock.tick(20)

	ev = pygame.event.poll()
	if ev.type == pygame.QUIT:
		running = False
	elif ev.type == pygame.KEYDOWN:
		if ev.key == pygame.K_w or ev.key == pygame.K_UP:
			if not renderer.game_paused:
				if model.lives > 0 and model.level_timer <= 0:
					model.ship.speed = ship_speed
					model.ship.facing = "n"
		elif ev.key == pygame.K_s or ev.key == pygame.K_DOWN:
			if not renderer.game_paused:
				if model.lives > 0 and model.level_timer <= 0:
					model.ship.speed = ship_speed
					model.ship.facing = "s"
		elif ev.key == pygame.K_a or ev.key == pygame.K_LEFT:
			if not renderer.game_paused:
				if model.lives > 0 and model.level_timer <= 0:
					model.ship.speed = ship_speed
					model.ship.facing = "w"
		elif ev.key == pygame.K_d or ev.key == pygame.K_RIGHT:
			if not renderer.game_paused:
				if model.lives > 0 and model.level_timer <= 0:
					model.ship.speed = ship_speed
					model.ship.facing = "e"
		elif ev.key == pygame.K_l:
			if model.death_timer > 0 or model.level_timer > 0:
				pass
			elif model.level == 0 or model.target > 0:
				pass
			else:
				model.level_complete()
		elif ev.key == pygame.K_ESCAPE:
			running = False
		elif ev.key == pygame.K_p:
			if model.death_timer > 0 or model.level_timer > 0:
				pass
			elif model.level == 0:
				if not muted: thunk_sound.play()
				model.lives = initial_lives
				model.score = 0
				set_game_speed(model.game_speed)
				model.init_level(1, levels[0])
			else:
				renderer.game_paused = not renderer.game_paused
		elif ev.key == pygame.K_q:
			if model.level > 0:
				model.level = 0
			else:
				running = False
		elif ev.key == pygame.K_MINUS:
			if model.level == 0 and model.game_speed > 1:
				model.game_speed -= 1
				if not muted: ding_sound.play()
		elif ev.key == pygame.K_EQUALS:
			if model.level == 0 and model.game_speed < 5:
				model.game_speed += 1
				if not muted: ding_sound.play()
		elif ev.key == pygame.K_m:
			muted = not muted
	elif ev.type == pygame.JOYHATMOTION:
		x, y = joystick.get_hat(0);
		if y == 1:
			if not renderer.game_paused:
				if model.lives > 0 and model.level_timer <= 0:
					model.ship.speed = ship_speed
					model.ship.facing = "n"
		elif y == -1:
			if not renderer.game_paused:
				if model.lives > 0 and model.level_timer <= 0:
					model.ship.speed = ship_speed
					model.ship.facing = "s"
		elif x == -1:
			if not renderer.game_paused:
				if model.lives > 0 and model.level_timer <= 0:
					model.ship.speed = ship_speed
					model.ship.facing = "w"
		elif x == 1:
			if not renderer.game_paused:
				if model.lives > 0 and model.level_timer <= 0:
					model.ship.speed = ship_speed
					model.ship.facing = "e"
	elif ev.type == pygame.JOYBUTTONDOWN:
		if buttons > 0 and joystick.get_button(0):
			if model.death_timer > 0 or model.level_timer > 0:
				pass
			elif model.level == 0 or model.target > 0:
				pass
			else:
				model.level_complete()
		elif buttons > 1 and joystick.get_button(1):
			if model.death_timer > 0 or model.level_timer > 0:
				pass
			elif model.level == 0:
				if not muted: thunk_sound.play()
				model.lives = initial_lives
				model.score = 0
				set_game_speed(model.game_speed)
				model.init_level(1, levels[0])
			else:
				renderer.game_paused = not renderer.game_paused
			
	# Simulations need the time in seconds, dammit!
	if model.level > 0 and not renderer.game_paused:
		model.update(delta_t * 0.001)
	renderer.render()
